
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include "cmd_lib.h"

static int filelength(FILE* pHandle);

/*
=================
=
= Error
=
= For abnormal program terminations
=
=================
*/
void Error(const char* error, ...)
{
	va_list	argptr;

	va_start(argptr, error);
	vprintf(error, argptr);
	va_end(argptr);
	printf("\n");
	exit(EXIT_FAILURE);
}

/*
==============
=
= SafeMalloc
=
==============
*/
void *SafeMalloc(long size)
{
	void* ptr;

	ptr = (void*)malloc(size);

	if (NULL == ptr)
	{
	    Error("Malloc failure for %lu bytes", size);
	}

	return ptr;
}

/*
==============
=
= SafeRead
=
==============
*/
void SafeRead(FILE* pHandle, void* buffer, long count)
{
	if(count != fread(buffer, 1, count, pHandle))
	{
		Error("File read failure");
	}
}

/*
==============
=
= SafeWrite
=
==============
*/
void SafeWrite(FILE* pHandle, void* buffer, long count)
{
	if (count != fwrite(buffer, 1, count, pHandle))
	{
		Error("File write failure");
	}
}

/*
==============
=
= SafeOpenRead
=
==============
*/
FILE* SafeOpenRead(char* filename)
{
	FILE* pHandle;

	pHandle = fopen(filename, "rb");

	if (NULL == pHandle)
	{
		Error("Error opening %s: %s", filename, strerror(errno));
	}

	return pHandle;
}


/*
==============
=
= SafeOpenWrite
=
==============
*/
FILE* SafeOpenWrite(char* filename)
{
	FILE* pHandle;

	pHandle = fopen(filename, "wb");

	if (NULL == pHandle)
	{
		Error("Error opening %s: %s", filename, strerror(errno));
	}

	return pHandle;
}


/*
==============
=
= LoadFile
=
==============
*/
long LoadFile(char* filename, void** bufferptr)
{
	FILE *pHandle;
	long	length;
	void *buffer;

	pHandle = SafeOpenRead(filename);
	length = filelength(pHandle);
	buffer = SafeMalloc(length);
	SafeRead(pHandle, buffer, length);
	fclose(pHandle);

	*bufferptr = buffer;
	return length;
}


/*
================
=
= filelength
=
================
*/
static int filelength(FILE* pHandle)
{
	int size;

	fseek(pHandle, 0, SEEK_END);
	size = ftell(pHandle);
	rewind(pHandle);

	return size;
}
