
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "cmd_lib.h"
#include "ted_maps.h"

typedef struct
{
#pragma pack(push, 1)
	short		  RLEWtag;
	long		  headeroffsets[100];
	unsigned char tileinfo[1];
#pragma pack(pop)
} mapfiletype;

//========================================

static mapfiletype *tinf;
static FILE *maphandle;

maptype header;
unsigned short* mapplanes[3];

//========================================

static void CA_RLEWexpand(unsigned short* source, unsigned short* dest, long length, unsigned rlewtag);

/*
======================
=
= LoadTedHeader
=
======================
*/

void LoadTedHeader(const char* extension)
{
	char    name[200];

	/* load maphead.ext(offsets and tileinfo for map file) */
	strcpy(name, "maphead.");
	strcat(name, extension);
	LoadFile(name, (void**)&tinf);

	/* open the data file */
	strcpy(name, "maptemp.");
	strcat(name, extension);
	maphandle = SafeOpenRead(name);
}

/*
======================
=
= LoadMap
=
======================
*/

void LoadMap(int mapnum)
{
	long pos, compressed, expanded;
	int	 plane;
	unsigned char *buffer;

	/* load map header */
	pos = tinf->headeroffsets[mapnum];

	if (pos < 0)	// $FFFFFFFF start is a sparse map
	{
		Error("LoadMap: Tried to load a non existent map!");
	}

	fseek(maphandle, pos, SEEK_SET);
	SafeRead(maphandle, &header, sizeof(maptype));

	/* load the planes in */
	expanded = header.width * header.height * 2;

	for(plane = 0; plane <= 2; plane++)
	{
		pos = header.planestart[plane];
		fseek(maphandle, pos, SEEK_SET);

		compressed = header.planelength[plane];
		buffer = (unsigned char *)SafeMalloc(compressed);
		SafeRead(maphandle, buffer, compressed);

		mapplanes[plane] = (unsigned short*)SafeMalloc(expanded);

		/* unRLEW, skipping expanded length */
		CA_RLEWexpand((unsigned short*)(buffer + 2), mapplanes[plane], expanded >> 1, 0xabcd);

		free(buffer);
	}
}

/*
======================
=
= CA_RLEWexpand
= length is EXPANDED length
=
======================
*/
static void CA_RLEWexpand(unsigned short* source, unsigned short* dest, long length, unsigned rlewtag)
{
	unsigned short 	value, count, i;

	unsigned short* end;

	end = dest + length;
	//
	// expand it
	//
	do
	{
		value = *source++;
		if (value != rlewtag)
			//
			// uncompressed
			//
			*dest++ = value;
		else
		{
			//
			// compressed string
			//
			count = *source++;
			value = *source++;
			for (i = 1; i <= count; i++)
				*dest++ = value;
		}
	} while (dest < end);
}